"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeExperimentModel = exports.writeServiceConfigurationModel = exports.writeParticipantModel = exports.writePeerconnectionModel = exports.writeRoleModel = exports.writeDeviceModel = void 0;
const errors_1 = require("../../types/errors");
const create_1 = require("./create");
const HOUR = 60 * 60 * 1000;
/**
 * This function writes the data of a Device to a DeviceModel.
 * @param deviceModel The DeviceModel the data should be written to.
 * @param device The Device providing the data to be written.
 */
function writeDeviceModel(requestHandler, deviceModel, device) {
    if (!device.device)
        requestHandler.throw(errors_1.MissingPropertyError, 'Device is missing property "url"', 400);
    if (device.device)
        deviceModel.url = device.device;
    if (device.role)
        deviceModel.role = device.role;
}
exports.writeDeviceModel = writeDeviceModel;
/**
 * This function writes the data of a Role to a RoleModel.
 * @param roleModel The RoleModel the data should be written to.
 * @param role The Role providing the data to be written.
 */
function writeRoleModel(_requestHandler, roleModel, role) {
    if (role.name)
        roleModel.name = role.name;
    if (role.description)
        roleModel.description = role.description;
}
exports.writeRoleModel = writeRoleModel;
/**
 * This function writes the data of a string to a PeerconnectionModel.
 * @param peerconnectionModel The PeerconnectionModel the data should be written to.
 * @param peerconnectionUrl The url of a peerconnection.
 */
function writePeerconnectionModel(_requestHandler, peerconnectionModel, peerconnectionUrl) {
    peerconnectionModel.url = peerconnectionUrl;
}
exports.writePeerconnectionModel = writePeerconnectionModel;
/**
 * This function writes the data of a Participant to a ParticipantModel.
 * @param participantModel The ParticipantModel the data should be written to.
 * @param participant The Participant providing the data to be written.
 */
function writeParticipantModel(_requestHandler, participantModel, participant) {
    if (participant.role)
        participantModel.role = participant.role;
    if (participant.serviceId)
        participantModel.serviceId = participant.serviceId;
    if (participant.config)
        participantModel.config = participant.config;
}
exports.writeParticipantModel = writeParticipantModel;
/**
 * This function writes the data of a ServiceConfiguration to a ServiceConfigurationModel.
 * @param serviceConfigurationModel The ServiceConfigurationModel the data should be written to.
 * @param serviceConfiguration The ServiceConfiguration providing the data to be written.
 */
function writeServiceConfigurationModel(requestHandler, serviceConfigurationModel, serviceConfiguration) {
    if (serviceConfiguration.serviceType)
        serviceConfigurationModel.serviceType = serviceConfiguration.serviceType;
    if (serviceConfiguration.configuration)
        serviceConfigurationModel.configuration = serviceConfiguration.configuration;
    if (serviceConfiguration.participants) {
        serviceConfigurationModel.participants = [];
        for (const participant of serviceConfiguration.participants) {
            const participantModel = requestHandler.executeSync(create_1.createParticipantModel, participant);
            serviceConfigurationModel.participants.push(participantModel);
        }
    }
}
exports.writeServiceConfigurationModel = writeServiceConfigurationModel;
/**
 * This function writes the data of a Experiment to a ExperimentModel.
 * @param experimentModel The ExperimentModel the data should be written to.
 * @param experiment The Experiment providing the data to be written.
 */
function writeExperimentModel(requestHandler, experimentModel, experiment) {
    experimentModel.status = experiment.status ?? experimentModel.status ?? 'created';
    if (experiment.bookingTime) {
        if (experiment.bookingTime.startTime)
            experimentModel.bookingStart = experiment.bookingTime.startTime;
        if (experiment.bookingTime.endTime)
            experimentModel.bookingEnd = experiment.bookingTime.endTime;
    }
    else {
        const start = Date.now();
        const end = start + HOUR;
        experimentModel.bookingStart =
            experimentModel.bookingStart ?? new Date(start).toISOString();
        experimentModel.bookingEnd =
            experimentModel.bookingEnd ?? new Date(end).toISOString();
    }
    if (experiment.devices) {
        experimentModel.devices = [];
        for (const device of experiment.devices) {
            const deviceModel = requestHandler.executeSync(create_1.createDeviceModel, device);
            experimentModel.devices.push(deviceModel);
        }
    }
    if (experiment.roles) {
        experimentModel.roles = [];
        for (const role of experiment.roles) {
            const roleModel = requestHandler.executeSync(create_1.createRoleModel, role);
            experimentModel.roles.push(roleModel);
        }
    }
    if (experiment.connections) {
        experimentModel.connections = [];
        for (const peerconnection of experiment.connections) {
            const peerconnectionModel = requestHandler.executeSync(create_1.createPeerconnectionModel, peerconnection);
            experimentModel.connections.push(peerconnectionModel);
        }
    }
    if (experiment.serviceConfigurations) {
        experimentModel.serviceConfigurations = [];
        for (const serviceConfiguration of experiment.serviceConfigurations) {
            const serviceConfigurationModel = requestHandler.executeSync(create_1.createServiceConfigurationModel, serviceConfiguration);
            experimentModel.serviceConfigurations.push(serviceConfigurationModel);
        }
    }
}
exports.writeExperimentModel = writeExperimentModel;
//# sourceMappingURL=write.js.map