"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.patchExperimentsByExperimentId = exports.deleteExperimentsByExperimentId = exports.getExperimentsByExperimentId = exports.postExperiments = exports.getExperiments = void 0;
const format_1 = require("../database/methods/format");
const write_1 = require("../database/methods/write");
const experimentStatus_1 = require("../util/experimentStatus");
const errors_1 = require("../types/errors");
const find_1 = require("../database/methods/find");
const create_1 = require("../database/methods/create");
const save_1 = require("../database/methods/save");
const delete_1 = require("../database/methods/delete");
const requestHandler_1 = require("../util/requestHandler");
/**
 * This function implements the functionality for handling GET requests on
 * /experiments endpoint.
 * @param _user The user submitting the request.
 */
const getExperiments = async (_user) => {
    const requestHandler = new requestHandler_1.RequestHandler('getExperiments');
    requestHandler.log('info', `Handling GET request on endpoint /experiments`);
    const experiments = await requestHandler.executeAsync(find_1.findAllExperimentModels);
    const result = {
        status: 200,
        body: experiments.map((e) => requestHandler.executeSync(format_1.formatExperimentModel, e)),
    };
    requestHandler.log('info', `Successfully handled GET request on endpoint /experiments`);
    return result;
};
exports.getExperiments = getExperiments;
/**
 * This function implements the functionality for handling POST requests on
 * /experiments endpoint.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 */
const postExperiments = async (body, _user) => {
    const requestHandler = new requestHandler_1.RequestHandler('postExperiments');
    requestHandler.log('info', `Handling POST request on endpoint /experiments`);
    const experimentModel = requestHandler.executeSync(create_1.createExperimentModel, body);
    const requestedStatus = body.status;
    await requestHandler.executeAsync(save_1.saveExperimentModel, experimentModel);
    if (requestedStatus === 'booked')
        await requestHandler.executeAsync(experimentStatus_1.bookExperiment, experimentModel);
    if (requestedStatus === 'running')
        await requestHandler.executeAsync(experimentStatus_1.runExperiment, experimentModel);
    if (requestedStatus === 'finished')
        await requestHandler.executeAsync(experimentStatus_1.finishExperiment, experimentModel);
    await requestHandler.executeAsync(save_1.saveExperimentModel, experimentModel); // NOTE: truly needed?
    const result = {
        status: 201,
        body: requestHandler.executeSync(format_1.formatExperimentModel, experimentModel),
    };
    requestHandler.log('info', `Successfully handled POST request on endpoint /experiments`);
    return result;
};
exports.postExperiments = postExperiments;
/**
 * This function implements the functionality for handling GET requests on
 * /experiments/{experiment_id} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 */
const getExperimentsByExperimentId = async (parameters, _user) => {
    const requestHandler = new requestHandler_1.RequestHandler('getExperimentsByExperimentId');
    requestHandler.log('info', `Handling GET request on endpoint /experiments/${parameters.experiment_id}`);
    const experimentModel = await requestHandler.executeAsync(find_1.findExperimentModelById, parameters.experiment_id);
    if (!experimentModel) {
        requestHandler.throw(errors_1.MissingEntityError, `Could not find experiment model ${parameters.experiment_id}`, 404);
    }
    const result = {
        status: 200,
        body: requestHandler.executeSync(format_1.formatExperimentModel, experimentModel),
    };
    requestHandler.log('info', `Successfully handled GET request on endpoint /experiments/${parameters.experiment_id}`);
    return result;
};
exports.getExperimentsByExperimentId = getExperimentsByExperimentId;
/**
 * This function implements the functionality for handling DELETE requests on
 * /experiments/{experiment_id} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 */
const deleteExperimentsByExperimentId = async (parameters, _user) => {
    const requestHandler = new requestHandler_1.RequestHandler('deleteExperimentsByExperimentId');
    requestHandler.log('info', `Handling DELETE request on endpoint /experiments/${parameters.experiment_id}`);
    const result = await requestHandler.executeAsync(delete_1.deleteExperimentModelById, parameters.experiment_id);
    if (!result.affected) {
        requestHandler.throw(errors_1.MissingEntityError, `Could not find experiment model ${parameters.experiment_id}`, 404);
    }
    if (result.affected > 1) {
        requestHandler.throw(errors_1.InconsistentDatabaseError, `More than one experiment model with id ${parameters.experiment_id} deleted`, 500);
    }
    requestHandler.log('info', `Successfully handled DELETE request on endpoint /experiments/${parameters.experiment_id}`);
    return {
        status: 204,
    };
};
exports.deleteExperimentsByExperimentId = deleteExperimentsByExperimentId;
/**
 * This function implements the functionality for handling PATCH requests on
 * /experiment/{experiment_id} endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 */
const patchExperimentsByExperimentId = async (parameters, body, _user) => {
    const requestHandler = new requestHandler_1.RequestHandler('patchExperimentsByExperimentId');
    requestHandler.log('info', `Handling PATCH request on endpoint /experiments/${parameters.experiment_id}`);
    const experimentModel = await requestHandler.executeAsync(find_1.findExperimentModelById, parameters.experiment_id);
    if (!experimentModel) {
        requestHandler.throw(errors_1.MissingEntityError, `Could not find experiment model ${parameters.experiment_id}`, 404);
    }
    if (body)
        requestHandler.executeSync(write_1.writeExperimentModel, experimentModel, body);
    if (experimentModel.status === 'booked')
        await requestHandler.executeAsync(experimentStatus_1.bookExperiment, experimentModel);
    if (experimentModel.status === 'running')
        await requestHandler.executeAsync(experimentStatus_1.runExperiment, experimentModel);
    if (experimentModel.status === 'finished')
        await requestHandler.executeAsync(experimentStatus_1.finishExperiment, experimentModel);
    await requestHandler.executeAsync(save_1.saveExperimentModel, experimentModel);
    const result = {
        status: 200,
        body: requestHandler.executeSync(format_1.formatExperimentModel, experimentModel),
    };
    requestHandler.log('info', `Successfully handled PATCH request on endpoint /experiments/${parameters.experiment_id}`);
    return result;
};
exports.patchExperimentsByExperimentId = patchExperimentsByExperimentId;
//# sourceMappingURL=experiments.js.map